@@ -14,7 +14,11 @@ import warnings
 from sunpy.net import Fido
 from sunpy.net import attrs as a
 
-from seppy.util import resample_df
+from seppy.util import custom_notification, custom_warning, resample_df
+
+
+logger = pooch.get_logger()
+logger.setLevel("WARNING")
 
 
 def _download_metafile(dataset, path=None):
@@ -30,6 +34,7 @@ def _download_metafile(dataset, path=None):
         downloaded_file = pooch.retrieve(url=url, known_hash=None, fname=fname, path=path, progressbar=True)
     except ModuleNotFoundError:
         downloaded_file = pooch.retrieve(url=url, known_hash=None, fname=fname, path=path, progressbar=False)
+    #
     return downloaded_file
 
 
@@ -39,23 +44,19 @@ def wind3dp_download_fido(dataset, startdate, enddate, path=None, max_conn=5):
 
     Parameters
     ----------
-    dataset : {str}
-        Name of Wind/3DP dataset:
-        - 'WI_SFSP_3DP': Electron omnidirectional fluxes 27 keV - 520 keV, often
-            at 24 sec
-        - 'WI_SFPD_3DP': Electron energy-angle distributions 27 keV to 520 keV,
-            often at 24 sec
-        - 'WI_SOSP_3DP': Proton omnidirectional fluxes 70 keV - 6.8 MeV, often
-            at 24 sec
-        - 'WI_SOPD_3DP': Proton energy-angle distributions 70 keV - 6.8 MeV,
-            often at 24 sec
-    startdate, enddate : {datetime or str}
+    dataset : str
+        Name of Wind/3DP dataset: \n
+            - 'WI_SFSP_3DP': Electron omnidirectional fluxes 27 keV - 520 keV, often at 24 sec \n
+            - 'WI_SFPD_3DP': Electron energy-angle distributions 27 keV to 520 keV, often at 24 sec \n
+            - 'WI_SOSP_3DP': Proton omnidirectional fluxes 70 keV - 6.8 MeV, often at 24 sec \n
+            - 'WI_SOPD_3DP': Proton energy-angle distributions 70 keV - 6.8 MeV, often at 24 sec \n
+    startdate, enddate : datetime or str
         Datetime object (e.g., dt.date(2021,12,31) or dt.datetime(2021,4,15)) or
         "standard" datetime string (e.g., "2021/04/15") (enddate must always be
         later than startdate)
-    path : {str}, optional
+    path : str, optional
         Local path for storing downloaded data, by default None
-    max_conn : {int}, optional
+    max_conn : int, optional
         The number of parallel download slots used by Fido.fetch, by default 5
 
     Returns
@@ -123,6 +124,7 @@ def wind3dp_single_download(file, path=None):
     except requests.HTTPError:
         print(f'No corresponding data found at {url}')
         downloaded_file = []
+    print('')
 
     return downloaded_file
 
@@ -134,15 +136,11 @@ def wind3dp_download(dataset, startdate, enddate, path=None, **kwargs):
     Parameters
     ----------
     dataset : {str}
-        Name of Wind/3DP dataset:
-        - 'WI_SFSP_3DP': Electron omnidirectional fluxes 27 keV - 520 keV, often
-            at 24 sec
-        - 'WI_SFPD_3DP': Electron energy-angle distributions 27 keV to 520 keV,
-            often at 24 sec
-        - 'WI_SOSP_3DP': Proton omnidirectional fluxes 70 keV - 6.8 MeV, often
-            at 24 sec
-        - 'WI_SOPD_3DP': Proton energy-angle distributions 70 keV - 6.8 MeV,
-            often at 24 sec
+        Name of Wind/3DP dataset: \n
+            - 'WI_SFSP_3DP': Electron omnidirectional fluxes 27 keV - 520 keV, often at 24 sec \n
+            - 'WI_SFPD_3DP': Electron energy-angle distributions 27 keV to 520 keV, often at 24 sec \n
+            - 'WI_SOSP_3DP': Proton omnidirectional fluxes 70 keV - 6.8 MeV, often at 24 sec \n
+            - 'WI_SOPD_3DP': Proton energy-angle distributions 70 keV - 6.8 MeV, often at 24 sec
     startdate, enddate : {datetime or str}
         Datetime object (e.g., dt.date(2021,12,31) or dt.datetime(2021,4,15)) or
         "standard" datetime string (e.g., "2021/04/15") (enddate must always be
@@ -174,6 +172,9 @@ def wind3dp_download(dataset, startdate, enddate, path=None, **kwargs):
             if not os.path.exists(f):
                 # downloaded_file = Fido.fetch(result[0][i], path=path, max_conn=max_conn)
                 downloaded_file = wind3dp_single_download(files[i], path=path)
+                if downloaded_file == []:
+                    print('Trying download from CDAWeb...')
+                    downloaded_file = Fido.fetch(result[0][i], path=path)  #, max_conn=max_conn)
 
     except (RuntimeError, IndexError):
         print(f'Unable to obtain "{dataset}" data for {startdate}-{enddate}!')
@@ -232,29 +233,29 @@ def wind3dp_load(dataset, startdate, enddate, resample="1min", multi_index=True,
 
     Parameters
     ----------
-    dataset : {str}
-        Name of Wind/3DP dataset:
-        - 'WI_SFSP_3DP': Electron omnidirectional fluxes 27 keV - 520 keV, often
-            at 24 sec
-        - 'WI_SFPD_3DP': Electron energy-angle distributions 27 keV to 520 keV,
-            often at 24 sec
-        - 'WI_SOSP_3DP': Proton omnidirectional fluxes 70 keV - 6.8 MeV, often
-            at 24 sec
-        - 'WI_SOPD_3DP': Proton energy-angle distributions 70 keV - 6.8 MeV,
-            often at 24 sec
-    startdate, enddate : {datetime or str}
+    dataset : str
+        Name of Wind/3DP dataset: \n
+            - 'WI_SFSP_3DP': Electron omnidirectional fluxes 27 keV - 520 keV, often at 24 sec \n
+            - 'WI_SFPD_3DP': Electron energy-angle distributions 27 keV to 520 keV, often at 24 sec \n
+            - 'WI_SOSP_3DP': Proton omnidirectional fluxes 70 keV - 6.8 MeV, often at 24 sec \n
+            - 'WI_SOPD_3DP': Proton energy-angle distributions 70 keV - 6.8 MeV, often at 24 sec \n
+    startdate, enddate : datetime or str
         Datetime object (e.g., dt.date(2021,12,31) or dt.datetime(2021,4,15)) or
         "standard" datetime string (e.g., "2021/04/15") (enddate must always be
         later than startdate)
-    resample : {str}, optional
+    resample : str, optional
         Frequency to which the original data (~24 seconds) is resamepled. Pandas
-        frequency (e.g., '1min' or '1h') or None, by default "1min"
-    multi_index : {bool}, optional
+        frequency (e.g., '1min' or '1h') or None, by default "1min".
+        Note that this is just a simple wrapper around thepandas
+        resample function that is calculating the mean of the data in the new
+        time bins. This is not necessarily the correct way to resample data,
+        depending on the data type (for example for errors)!
+    multi_index : bool, optional
         Provide output for pitch-angle resolved data as Pandas Dataframe with
         multiindex, by default True
-    path : {str}, optional
+    path : str, optional
         Local path for storing downloaded data, by default None
-    threshold : {int or float}, optional
+    threshold : int or float, optional
         Replace all FLUX values above 'threshold' with np.nan, by default None
 
     Returns
@@ -273,22 +274,36 @@ def wind3dp_load(dataset, startdate, enddate, resample="1min", multi_index=True,
         metacdf = cdflib.CDF(path_to_metafile)
 
         e_mean = df.filter(like='ENERGY_').mean()
-        # ∼30% ΔE/E => ΔE = 0.3*E
         # from Table 3 of Wilson et al. 2021, https://doi.org/10.1029/2020RG000714
-        delta_e = 0.3 * e_mean
+        # ∼30% ΔE/E => ΔE = 0.3*E
+        if dataset in ['WI_SFSP_3DP', 'WI_SFPD_3DP', 'WI_SOSP_3DP', 'WI_SOPD_3DP']:
+            delta_e = 0.3 * e_mean
+        # ∼20% ΔE/E => ΔE = 0.2*E
+        elif dataset in ['WI_ELSP_3DP', 'WI_ELPD_3DP', 'WI_EHSP_3DP', 'WI_EHPD_3DP']:
+            delta_e = 0.2 * e_mean
         e_low = e_mean - delta_e
         e_high = e_mean + delta_e
-        energies = pd.concat([e_mean, delta_e, e_low, e_high], axis=1, keys=['mean_E', 'DE', 'lower_E', 'upper_E'])
+        energies = pd.concat([e_mean, delta_e, 2*delta_e, e_low, e_high], axis=1, keys=['mean_E', 'delta_e', 'DE', 'lower_E', 'upper_E'])
         energies['Bins_Text']= np.around(e_low/1e3, 2).astype('string') +' - '+ np.around(e_high/1e3, 2).astype('string') + ' keV'
 
+        for col in ['mean_E', 'delta_e', 'DE', 'lower_E', 'upper_E']:
+            energies[col] = energies[col]/1e6  # convert energies to MeV
+
+        custom_warning('The energy values in the metadata are now provided as MeV (previously eV)! Also DE is now 2*delta_e (previously just delta_e). Please adapt your scripts accordingly!')
+
         meta = {'channels_dict_df': energies,
-                'APPROX_ENERGY_LABELS': metacdf.varget('APPROX_ENERGY_LABELS'),
                 'ENERGY_UNITS': metacdf.varattsget('ENERGY')['UNITS'],
                 'FLUX_UNITS': metacdf.varattsget('FLUX')['UNITS'],
                 'FLUX_FILLVAL': metacdf.varattsget('FLUX')['FILLVAL'],
-                'FLUX_LABELS': metacdf.varget('FLUX_ENERGY_LABL'),
                 }
 
+        # for SFSP, SOSP, SFPD, SFSP:
+        try:
+            meta['APPROX_ENERGY_LABELS'] = metacdf.varget('APPROX_ENERGY_LABELS')
+            meta['FLUX_LABELS'] = metacdf.varget('FLUX_ENERGY_LABL')
+        except:
+            pass
+
         # create multi-index data frame of flux
         if multi_index:
             if dataset == 'WI_SFPD_3DP' or dataset == 'WI_SOPD_3DP':
